unit DynDb;

interface

uses
  Classes, Db, TypInfo, SysUtils;

{global support functions, from DDH book, modified}

function IsDataAware (DbComp: TComponent): Boolean;
procedure ConnectDataSource (DbComp: TComponent;
  DataSource: TDataSource);
function HasDataField (DbComp: TComponent): Boolean;
procedure ConnectDataField (DbComp: TComponent;
  FieldName: string);

implementation

function IsDataAware (DbComp: TComponent): Boolean;
var
  PropInfo: PPropInfo;
begin
  if not Assigned (DbComp) then
    raise Exception.Create (
      'Invalid Component in "IsDataAware"');

  // check the DataSource property
  PropInfo := GetPropInfo (
    DbComp.ClassInfo, 'DataSource');
  Result := (PropInfo <> nil);
end;

procedure ConnectDataSource (DbComp: TComponent;
  DataSource: TDataSource);
var
  PropInfo: PPropInfo;
begin
  if not Assigned (DbComp) then
    raise Exception.Create (
      'Invalid Component in "ConnectDataSource"');

  // set the DataSource property
  PropInfo := GetPropInfo (
    DbComp.ClassInfo, 'DataSource');
  if PropInfo = nil then
    raise Exception.Create (
      '"ConnectDataSource" failed: Missing DataSource property');

  // connect
  SetOrdProp (DbComp, PropInfo,
    Integer (Pointer (DataSource)));
end;

function HasDataField (DbComp: TComponent): Boolean;
var
  PropInfo: PPropInfo;
begin
  if not Assigned (DbComp) then
    raise Exception.Create (
      'Invalid Component in "HasDataField"');

  // check the DataSource property
  PropInfo := GetPropInfo (
    DbComp.ClassInfo, 'DataField');
  Result := (PropInfo <> nil);
end;


procedure ConnectDataField (DbComp: TComponent;
  FieldName: string);
var
  PropInfo: PPropInfo;
begin
  if not Assigned (DbComp) then
    raise Exception.Create (
      'Invalid Component in "ConnectDataField"');

  // set the DataField property
  PropInfo := GetPropInfo (
    DbComp.ClassInfo, 'DataField');
  if PropInfo = nil then
    raise Exception.Create (
      '"ConnectDataField" failed: Missing DataField property');

  // connect
  SetStrProp (DbComp, PropInfo, FieldName);
end;

end.
