unit EverythingButTheKitchenSink;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Db, DbTables, ExtCtrls;

type
  TEverythingButTheKitchenSink = class(TCustomControl)
  private
    fDataSet: TDataSet;
    fDataSource: TDataSource;
    fCompsToDoList: TList;
    fTimer: TTimer;
    fPages: TPageControl;
    Ffilename: string;
    image: TImage;
    procedure SetActive(const Value: Boolean);
    procedure SetDatabaseName(const Value: string);
    procedure SetTableName(const Value: string);
    function GetActive: Boolean;
    function GetDatabaseName: string;
    function GetTableName: string;
    procedure Setfilename(const Value: string);
  protected
    procedure TimerProc (Sender: TObject);
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Notification (
      AComponent: TComponent; Operation: TOperation); override;
  published
    property DatabaseName: string read GetDatabaseName write SetDatabaseName;
    property TableName: string read GetTableName write SetTableName;
    property Active: Boolean read GetActive write SetActive;
    property filename: string read Ffilename write Setfilename;
  end;

procedure Register;

implementation

uses
  DynDb, dbctrls, dbgrids, stdctrls;

procedure Register;
begin
  RegisterComponents('FunSide', [TEverythingButTheKitchenSink]);
end;

{ TEverythingButTheKitchenSink }

constructor TEverythingButTheKitchenSink.Create(AOwner: TComponent);
var
  page: TTabSheet;
  edit: TDbEdit;
  i: integer;
begin
  inherited Create (AOwner);

  fDataSet := TTable.Create (self);
  (fDataSet as TTable).DatabaseName := 'DBDEMOS';
  (fDataSet as TTable).TableName := 'BIOLIFE.DB';
  fDataSet.Name := 'Internal';

  fDataSource := TDataSource.Create (self);
  fDataSource.DataSet := fDataSet;

  fTimer := TTimer.Create (self);
  fTimer.Enabled := False;
  fTimer.OnTimer := TimerProc;
  fCompsToDoList := TList.Create;

  fPages := TPageControl.Create (self);
  fPages.Parent := self;
  fPages.Align := alClient;

  SetBounds (0, 0, 300, 200);

  page := TTabSheet.Create (self);
  page.PageControl := fPages;
  page.Caption := 'Grid';
  with TDbGrid.Create (self) do
  begin
    Parent := page;
    Align := alClient;
    DataSource := fDataSource;
  end;

  page := TTabSheet.Create (self);
  page.PageControl := fPages;
  page.Align := alClient;
  page.Caption := 'Auto DbEdits';

  for i := 1 to 5 do
  begin
    edit := TDbEdit.Create (self);
    edit.Parent := page;
    edit.SetBounds (20, 20 + 30 * i,
        page.clientwidth - 40, 40 + 30 * i);
    // uses auto-hook technology for datasource / field
  end;

  page := TTabSheet.Create (self);
  page.PageControl := fPages;
  page.Align := alClient;
  page.Caption := 'Sink';

  image := TImage.Create (self);
  image.Parent := page;
  image.Align := alClient;
  image.Stretch := true;
end;

destructor TEverythingButTheKitchenSink.Destroy;
begin
  fCompsToDoList.Free;
  fTImer.Free;
  inherited Destroy;
end;

function TEverythingButTheKitchenSink.GetActive: Boolean;
begin
  if Assigned (fDataSet) then
    Result := fDataSet.Active
  else
    Result := False;
end;

function TEverythingButTheKitchenSink.GetDatabaseName: string;
begin
  if Assigned (fDataSet) and (fDataSet is TTable) then
    Result := (fDataSet as TTable).DatabaseName
  else
    Result := '';
end;

function TEverythingButTheKitchenSink.GetTableName: string;
begin
  if Assigned (fDataSet) and (fDataSet is TTable) then
    Result := (fDataSet as TTable).TableName
  else
    Result := '';
end;

procedure TEverythingButTheKitchenSink.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;

  if Assigned (AComponent) and
    (AComponent <> self) and
    (Operation = opInsert) and
    IsDataAware (AComponent) then
  begin
    fCompsToDoList.Add (AComponent);
    fTimer.Enabled := True;
  end;
end;

procedure TEverythingButTheKitchenSink.SetActive(const Value: Boolean);
begin
  fDataSet.Active := Value;
end;

procedure TEverythingButTheKitchenSink.SetDatabaseName(
  const Value: string);
begin
  (fDataSet as TTable).DatabaseName := Value;
end;

procedure TEverythingButTheKitchenSink.Setfilename(const Value: string);
begin
  Ffilename := Value;
  try
    image.Picture.LoadFromFile (value);
  except
    ffilename := '';
  end;
end;

procedure TEverythingButTheKitchenSink.SetTableName(const Value: string);
begin
  (fDataSet as TTable).TableName := Value;
end;

procedure TEverythingButTheKitchenSink.TimerProc(Sender: TObject);
var
  Comp: TComponent;
begin
  if fCompsToDoList.Count > 0 then
  begin
    Comp := fCompsToDoList [0];
    // set properties
    if IsDataAware (Comp) then
      ConnectDataSource (Comp, fDataSource);
    if HasDataField (Comp) and
        fDataSet.Active then
      ConnectDataField (Comp, fDataSet.Fields [Random (
          fDataSet.FieldCount)].FieldName);
    // remove component from list
    fCompsToDoList.Remove (Comp);
  end;
  if fCompsToDoList.Count = 0 then
    fTimer.Enabled := False;
end;

initialization
  randomize;

end.
